@@ -5,7 +5,7 @@ import sys
 import time
 from dataclasses import dataclass
 from types import TracebackType
-from typing import Any, Dict, Iterable, List, Optional, Tuple, Type
+from typing import Dict, Iterable, List, Optional, Tuple, Type
 
 import humanize
 from rich import box
@@ -20,7 +20,7 @@ from rich.text import Text, TextType
 from neuro_sdk import JobDescription, JobRestartPolicy, JobStatus, JobTelemetry
 
 from neuro_cli.formatters.utils import DatetimeFormatter
-from neuro_cli.parse_utils import JobColumnInfo
+from neuro_cli.parse_utils import JobColumnInfo, JobTelemetryKeyFunc
 from neuro_cli.utils import format_size
 
 from .utils import (
@@ -67,6 +67,23 @@ def _get_run_time(job: JobDescription) -> datetime.timedelta:
     return run_time
 
 
+def get_lifespan_ends(job: JobDescription) -> Optional[datetime.datetime]:
+    if (
+        job.status
+        in [
+            JobStatus.PENDING,
+            JobStatus.RUNNING,
+            JobStatus.SUSPENDED,
+        ]
+        and job.life_span
+    ):
+        life_span = datetime.timedelta(seconds=job.life_span)
+        runtime_left = life_span - _get_run_time(job)
+        runtime_ends = datetime.datetime.now(datetime.timezone.utc) + runtime_left
+        return runtime_ends
+    return None
+
+
 class JobStatusFormatter:
     def __init__(
         self, uri_formatter: URIFormatter, datetime_formatter: DatetimeFormatter
@@ -225,20 +242,10 @@ class JobStatusFormatter:
             table.add_row(
                 "Started", self._datetime_formatter(job_status.history.started_at)
             )
-        if (
-            job_status.status
-            in [
-                JobStatus.PENDING,
-                JobStatus.RUNNING,
-                JobStatus.SUSPENDED,
-            ]
-            and job_status.life_span
-        ):
-            life_span = datetime.timedelta(seconds=job_status.life_span)
-            runtime_left = life_span - _get_run_time(job_status)
-            runtime_ends = datetime.datetime.now(datetime.timezone.utc) + runtime_left
+        lifespan_ends = get_lifespan_ends(job_status)
+        if lifespan_ends:
             table.add_row(
-                "Life span ends (approx)", self._datetime_formatter(runtime_ends)
+                "Life span ends (approx)", self._datetime_formatter(lifespan_ends)
             )
         if job_status.status in [
             JobStatus.CANCELLED,
@@ -269,6 +276,23 @@ class JobStatusFormatter:
         return table
 
 
+class LifeSpanUpdateFormatter:
+    def __init__(self, datetime_formatter: DatetimeFormatter) -> None:
+        self._datetime_formatter = datetime_formatter
+
+    def __call__(self, job_status: JobDescription) -> RenderableType:
+        res = (
+            f"Updated job {job_status.id} life span to "
+            f"be {format_life_span(job_status.life_span)}"
+        )
+        life_span_ends = get_lifespan_ends(job_status)
+        if life_span_ends is not None:
+            res += (
+                f"\nLife span ends (approx): {self._datetime_formatter(life_span_ends)}"
+            )
+        return res
+
+
 def format_life_span(life_span: Optional[float]) -> str:
     if life_span is None:
         return ""
@@ -281,12 +305,20 @@ class JobTelemetryFormatter(RenderHook):
     def __init__(
         self,
         console: Console,
+        username: str,
+        sort_keys: List[Tuple[JobTelemetryKeyFunc, bool]],
+        columns: List[JobColumnInfo],
+        image_formatter: ImageFormatter,
         datetime_formatter: DatetimeFormatter,
         maxrows: Optional[int] = None,
     ) -> None:
         self._console = console
-        self._maxrows = maxrows
+        self._username = username
+        self.sort_keys = sort_keys
+        self._columns = columns
+        self._image_formatter = image_formatter
         self._datetime_formatter = datetime_formatter
+        self._maxrows = maxrows
         self._live_render = LiveRender(Table.grid())
         self._data: Dict[str, Tuple[JobDescription, JobTelemetry]] = {}
         self.changed = True
@@ -301,30 +333,36 @@ class JobTelemetryFormatter(RenderHook):
 
     def render(self) -> None:
         table = Table(box=box.SIMPLE_HEAVY)
-        table.add_column("ID", justify="left", width=40)
-        table.add_column("WHEN", justify="left", width=32)
-        table.add_column("CPU", justify="right", width=15)
-        table.add_column("MEMORY (MB)", justify="right", width=15)
-        table.add_column("GPU (%)", justify="right", width=15)
-        table.add_column("GPU_MEMORY (MB)", justify="right", width=15)
-
-        def sortkey(item: Tuple[JobDescription, JobTelemetry]) -> Any:
-            job, info = item
-            return info.cpu
+        _add_columns(table, self._columns)
 
         items = list(self._data.values())
-        items.sort(key=sortkey, reverse=True)
+        for keyfunc, reverse in reversed(self.sort_keys):
+            items.sort(key=keyfunc, reverse=reverse)
         maxrows = self._console.size.height - 4
         if self._maxrows is not None and self._maxrows < maxrows:
             maxrows = self._maxrows
         del items[max(maxrows, 1) :]
+
         for job, info in items:
-            created = self._datetime_formatter(job.history.created_at)
-            cpu = f"{info.cpu:.3f}"
-            mem = f"{info.memory:.3f}"
-            gpu = f"{info.gpu_duty_cycle}" if info.gpu_duty_cycle else "0"
-            gpu_mem = f"{info.gpu_memory:.3f}" if info.gpu_memory else "0"
-            table.add_row(job.id, created, cpu, mem, gpu, gpu_mem)
+            job_data = TabularJobRow.from_job(
+                job,
+                self._username,
+                image_formatter=self._image_formatter,
+                datetime_formatter=self._datetime_formatter,
+            )
+            telemetry_data = dict(
+                cpu=f"{info.cpu:.3f}",
+                memory=f"{info.memory:.3f}",
+                gpu=f"{info.gpu_duty_cycle}" if info.gpu_duty_cycle else "0",
+                gpu_memory=f"{info.gpu_memory:.3f}" if info.gpu_memory else "0",
+            )
+            fields = [
+                telemetry_data[column.id]
+                if column.id in telemetry_data
+                else getattr(job_data, column.id)
+                for column in self._columns
+            ]
+            table.add_row(*fields)
 
         if self._console.is_terminal:
             self._live_render.set_renderable(table)
@@ -404,19 +442,12 @@ class TabularJobRow:
         image_formatter: ImageFormatter,
         datetime_formatter: DatetimeFormatter,
     ) -> "TabularJobRow":
-        if job.status == JobStatus.PENDING:
-            when = job.history.created_at
-        elif job.status == JobStatus.RUNNING:
-            when = job.history.started_at
-        else:
-            when = job.history.finished_at
-        assert when is not None
         return cls(
             id=job.id,
             name=job.name if job.name else "",
             tags=",".join(job.tags),
             status=fmt_status(job.status),
-            when=datetime_formatter(when),
+            when=datetime_formatter(job.history.changed_at),
             created=datetime_formatter(job.history.created_at),
             started=datetime_formatter(job.history.started_at),
             finished=datetime_formatter(job.history.finished_at),
@@ -449,23 +480,7 @@ class TabularJobsFormatter(BaseJobsFormatter):
 
     def __call__(self, jobs: Iterable[JobDescription]) -> RenderableType:
         table = Table(box=box.SIMPLE_HEAVY)
-        column = self._columns[0]
-        table.add_column(
-            column.title,
-            style="bold",
-            justify=column.justify,
-            width=column.width,
-            min_width=column.min_width,
-            max_width=column.max_width,
-        )
-        for column in self._columns[1:]:
-            table.add_column(
-                column.title,
-                justify=column.justify,
-                width=column.width,
-                min_width=column.min_width,
-                max_width=column.max_width,
-            )
+        _add_columns(table, self._columns)
 
         for job in jobs:
             table.add_row(
@@ -525,6 +540,26 @@ class JobStartProgress:
         pass
 
 
+def _add_columns(table: Table, columns: List[JobColumnInfo]) -> None:
+    column = columns[0]
+    table.add_column(
+        column.title,
+        style="bold",
+        justify=column.justify,
+        width=column.width,
+        min_width=column.min_width,
+        max_width=column.max_width,
+    )
+    for column in columns[1:]:
+        table.add_column(
+            column.title,
+            justify=column.justify,
+            width=column.width,
+            min_width=column.min_width,
+            max_width=column.max_width,
+        )
+
+
 class DetailedJobStartProgress(JobStartProgress, RenderHook):
     def __init__(self, console: Console) -> None:
         self._time = self.time_factory()
