@@ -34,6 +34,7 @@ from neuro_cli.formatters.utils import (
     image_formatter,
     uri_formatter,
 )
+from neuro_cli.parse_utils import parse_sort_keys
 from neuro_cli.utils import resolve_disk
 
 from .ael import process_attach, process_exec, process_logs
@@ -43,7 +44,8 @@ from .click_types import (
     JOB_NAME,
     LOCAL_REMOTE_PORT,
     PRESET,
-    ImageType,
+    TOP_COLUMNS,
+    RemoteImageType,
 )
 from .const import EX_PLATFORMERROR
 from .formatters.jobs import (
@@ -51,14 +53,17 @@ from .formatters.jobs import (
     JobStartProgress,
     JobStatusFormatter,
     JobTelemetryFormatter,
+    LifeSpanUpdateFormatter,
     SimpleJobsFormatter,
     TabularJobsFormatter,
 )
 from .parse_utils import (
     JobColumnInfo,
-    get_default_columns,
-    parse_columns,
+    get_default_ps_columns,
+    get_default_top_columns,
+    parse_ps_columns,
     parse_timedelta,
+    parse_top_columns,
     serialize_timedelta,
 )
 from .root import Root
@@ -371,7 +376,7 @@ async def ls(
     neuro ps -t tag1 -t tag2
     """
 
-    format = await calc_columns(root.client, format)
+    format = await calc_ps_columns(root.client, format)
 
     statuses = calc_statuses(status, all)
     owners = set(owner)
@@ -458,6 +463,35 @@ async def status(root: Root, job: str, full_uri: bool) -> None:
     )
 
 
+@command()
+@argument("job", type=JOB)
+@argument(
+    "additional-life-span",
+    type=str,
+    metavar="TIMEDELTA",
+)
+async def bump_life_span(root: Root, job: str, additional_life_span: str) -> None:
+    """
+    Increase job life span
+    """
+    job_id = await resolve_job(
+        job,
+        client=root.client,
+        status=JobStatus.items(),
+    )
+    await root.client.jobs.bump_life_span(
+        id=job_id,
+        additional_life_span=parse_timedelta(additional_life_span).total_seconds(),
+    )
+    if not root.quiet:
+        res = await root.client.jobs.status(id=job_id)
+        root.print(
+            LifeSpanUpdateFormatter(
+                datetime_formatter=get_datetime_formatter(root.iso_datetime_format),
+            )(res)
+        )
+
+
 @command(deprecated=True, hidden=True)
 async def tags(root: Root) -> None:
     """
@@ -485,6 +519,63 @@ async def browse(root: Root, job: str) -> None:
 
 @command()
 @argument("jobs", nargs=-1, required=False, type=JOB)
+@option(
+    "-o",
+    "--owner",
+    multiple=True,
+    help="Filter out jobs by owner (multiple option). "
+    "Supports `ME` option to filter by the current user. "
+    "Specify `ALL` to show jobs of all users.",
+    secure=True,
+)
+@option("-n", "--name", metavar="NAME", help="Filter out jobs by name.", secure=True)
+@option(
+    "-t",
+    "--tag",
+    metavar="TAG",
+    type=str,
+    help="Filter out jobs by tag (multiple option)",
+    multiple=True,
+)
+@option(
+    "-d",
+    "--description",
+    metavar="DESCRIPTION",
+    default="",
+    help="Filter out jobs by description (exact match).",
+    secure=True,
+)
+@option(
+    "--since",
+    metavar="DATE",
+    help="Show jobs created after a specific date (including).",
+)
+@option(
+    "--until",
+    metavar="DATE",
+    help="Show jobs created before a specific date (including).",
+)
+@option(
+    "--sort",
+    metavar="COLUMNS",
+    help="Sort rows by specified column. "
+    'Add "-" prefix to revert the sorting order. '
+    "Multiple columns can be specified (comma separated).",
+    default="cpu",
+    show_default=True,
+)
+@option(
+    "--format",
+    type=TOP_COLUMNS,
+    help=(
+        'Output table format, see "neuro help top-format" '
+        "for more info about the format specification. "
+        "The default can be changed using the job.top-format "
+        'configuration variable documented in "neuro help user-config"'
+    ),
+    default=None,
+)
+@option("--full-uri", is_flag=True, help="Output full image URI.")
 @option(
     "--timeout",
     default=0,
@@ -492,14 +583,52 @@ async def browse(root: Root, job: str) -> None:
     show_default=True,
     help="Maximum allowed time for executing the command, 0 for no timeout",
 )
-async def top(root: Root, jobs: Sequence[str], timeout: float) -> None:
+async def top(
+    root: Root,
+    jobs: Sequence[str],
+    name: str,
+    tag: Sequence[str],
+    owner: Sequence[str],
+    since: str,
+    until: str,
+    description: str,
+    sort: str,
+    format: Optional[List[JobColumnInfo]],
+    full_uri: bool,
+    timeout: float,
+) -> None:
     """
     Display GPU/CPU/Memory usage.
-    """
-    observed: Set[str] = set()
 
-    async def create_pollers() -> None:
-        if jobs:
+    Examples:
+
+    neuro top
+    neuro top job-1 job-2
+    neuro top --owner=user-1 --owner=user-2
+    neuro top --name my-experiments-v1
+    neuro top -t tag1 -t tag2
+    """
+
+    sort_keys = parse_sort_keys(sort)
+    format = await calc_top_columns(root.client, format)
+
+    observed: Set[str] = set()
+    loop = asyncio.get_event_loop()
+
+    if jobs:
+        for opt, val in [
+            ("name", name),
+            ("owner", owner),
+            ("since", since),
+            ("until", until),
+            ("description", description),
+        ]:
+            if val:
+                raise click.UsageError(
+                    f"Option --{opt} is mutually exclusive with job arguments"
+                )
+
+        async def create_pollers() -> None:
             for job_str in jobs:
                 job_id = await resolve_job(
                     job_str, client=root.client, status=JobStatus.active_items()
@@ -508,35 +637,65 @@ async def top(root: Root, jobs: Sequence[str], timeout: float) -> None:
                     continue
                 observed.add(job_id)
                 job = await root.client.jobs.status(job_id)
-                asyncio.create_task(poller(job))
-        else:
-            since: Optional[datetime] = None
-            while True:
-                jobs2 = root.client.jobs.list(
-                    statuses=JobStatus.active_items(),
-                    owners=(root.client.username,),
-                    since=since,
+                loop.create_task(poller(job))
+
+    else:
+        owners = set(owner)
+        if not owners:
+            owners = {root.client.config.username}
+        elif "ALL" in owners:
+            owners.remove("ALL")
+            if owners:
+                raise click.UsageError(
+                    "Multiple --owner options are incompatible with --owner=ALL"
                 )
+        elif "ME" in owners:
+            owners.remove("ME")
+            owners.add(root.client.config.username)
+        tags = set(tag)
+        since_dt = _parse_date(since)
+        until_dt = _parse_date(until)
+
+        async def create_pollers() -> None:
+            nonlocal since_dt
+            while True:
+                jobs = root.client.jobs.list(
+                    statuses=JobStatus.active_items(),
+                    name=name,
+                    owners=owners,
+                    tags=tags,
+                    since=since_dt,
+                    until=until_dt,
+                )
+
+                # client-side filtering
+                if description:
+                    jobs = (job async for job in jobs if job.description == description)
+
                 dt: Optional[datetime]
                 dt = datetime.now(timezone.utc) - timedelta(minutes=1)
-                if since is None or since < dt:
-                    since = dt
-                async for job in jobs2:
+                if since_dt is None or since_dt < dt:
+                    since_dt = dt
+                async for job in jobs:
                     job_id = job.id
                     if job_id in observed:
                         continue
                     observed.add(job_id)
                     dt = job.history.created_at
-                    if dt is not None and since < dt:
-                        since = dt
-                    asyncio.create_task(poller(job))
+                    if dt is not None and since_dt < dt:
+                        since_dt = dt
+                    loop.create_task(poller(job))
                 await asyncio.sleep(TOP_NEW_JOBS_DELAY)
 
     async def poller(job: JobDescription) -> None:
-        async for info in root.client.jobs.top(job.id):
-            formatter.update(job, info)
-            await asyncio.sleep(0)
-        formatter.remove(job.id)
+        try:
+            async for info in root.client.jobs.top(job.id):
+                formatter.update(job, info)
+                await asyncio.sleep(0)
+        except ValueError:
+            pass  # Job is finished.
+        finally:
+            formatter.remove(job.id)
         await asyncio.sleep(0)
 
     async def renderer() -> None:
@@ -546,16 +705,30 @@ async def top(root: Root, jobs: Sequence[str], timeout: float) -> None:
                     formatter.render()
                 await asyncio.sleep(TOP_REFRESH_DELAY)
 
+    uri_fmtr: URIFormatter
+    if full_uri:
+        uri_fmtr = str
+    else:
+        uri_fmtr = uri_formatter(
+            username=root.client.username, cluster_name=root.client.cluster_name
+        )
+    image_fmtr = image_formatter(uri_formatter=uri_fmtr)
+    datetime_fmtr = get_datetime_formatter(root.iso_datetime_format)
+
     with JobTelemetryFormatter(
         root.console,
-        datetime_formatter=get_datetime_formatter(root.iso_datetime_format),
+        root.client.username,
+        sort_keys=sort_keys,
+        columns=format,
+        image_formatter=image_fmtr,
+        datetime_formatter=datetime_fmtr,
     ) as formatter:
         await asyncio.gather(create_pollers(), renderer())
 
 
 @command()
 @argument("job", type=JOB)
-@argument("image", type=ImageType())
+@argument("image", type=RemoteImageType())
 async def save(root: Root, job: str, image: RemoteImage) -> None:
     """
     Save job's state to an image.
@@ -603,7 +776,7 @@ async def kill(root: Root, jobs: Sequence[str]) -> None:
 
 
 @command(context_settings=dict(allow_interspersed_args=False))
-@argument("image", type=ImageType())
+@argument("image", type=RemoteImageType())
 @argument("cmd", nargs=-1, type=click.UNPROCESSED)
 @option(
     "-s",
@@ -908,6 +1081,7 @@ job.add_command(top)
 job.add_command(save)
 job.add_command(browse)
 job.add_command(attach)
+job.add_command(bump_life_span)
 
 
 job.add_command(alias(ls, "list", hidden=True))
@@ -1128,7 +1302,7 @@ def calc_statuses(status: Sequence[str], all: bool) -> Set[JobStatus]:
     return {JobStatus(s) for s in statuses}
 
 
-async def calc_columns(
+async def calc_ps_columns(
     client: Client, format: Optional[List[JobColumnInfo]]
 ) -> List[JobColumnInfo]:
     if format is None:
@@ -1137,8 +1311,22 @@ async def calc_columns(
         if section is not None:
             format_str = section.get("ps-format")
             if format_str is not None:
-                return parse_columns(format_str)
-        return get_default_columns()
+                return parse_ps_columns(format_str)
+        return get_default_ps_columns()
+    return format
+
+
+async def calc_top_columns(
+    client: Client, format: Optional[List[JobColumnInfo]]
+) -> List[JobColumnInfo]:
+    if format is None:
+        config = await client.config.get_user_config()
+        section = config.get("job")
+        if section is not None:
+            format_str = section.get("top-format")
+            if format_str is not None:
+                return parse_top_columns(format_str)
+        return get_default_top_columns()
     return format
 
 
