@@ -68,12 +68,14 @@ class Root:
     network_timeout: float
     config_path: Path
     trace: bool
+    force_trace_all: bool
     verbosity: int
     trace_hide_token: bool
     command_path: str
     command_params: List[Dict[str, Optional[str]]]
     skip_gmp_stats: bool
     show_traceback: bool
+    iso_datetime_format: bool
 
     _client: Optional[Client] = None
     _factory: Optional[Factory] = None
@@ -174,6 +176,7 @@ class Root:
                 path=self.config_path,
                 trace_configs=trace_configs,
                 trace_id=gen_trace_id(),
+                trace_sampled=True if self.force_trace_all else None,
             )
         return self._factory
 
