@@ -32,6 +32,7 @@ from neuro_cli.formatters.jobs import (
     JobStartProgress,
     JobStatusFormatter,
     JobTelemetryFormatter,
+    LifeSpanUpdateFormatter,
     SimpleJobsFormatter,
     TabularJobRow,
     TabularJobsFormatter,
@@ -42,7 +43,7 @@ from neuro_cli.formatters.utils import (
     image_formatter,
     uri_formatter,
 )
-from neuro_cli.parse_utils import parse_columns
+from neuro_cli.parse_utils import parse_ps_columns, parse_sort_keys, parse_top_columns
 
 TEST_JOB_ID = "job-ad09fe07-0c64-4d32-b477-3b737d215621"
 TEST_JOB_ID2 = "job-3f9c5f93-45be-4c5d-acbd-11c68260235f"
@@ -1156,12 +1157,17 @@ class TestJobOutputFormatter:
 class TestJobTelemetryFormatter:
     # Use utc timezone in test for stable constant result
 
-    def test_format_telemetry_line_no_gpu(
+    def test_format_telemetry_no_gpu(
         self, job_descr: JobDescription, rich_cmp: Any, new_console: _NewConsole
     ) -> None:
         console = new_console(tty=True, color=True)
         with JobTelemetryFormatter(
-            console, datetime_formatter=format_datetime_human
+            console,
+            "owner",
+            [],
+            parse_top_columns(None),
+            image_formatter=str,
+            datetime_formatter=format_datetime_human,
         ) as fmt:
             timestamp = 1_517_248_466.238_723_6
             telemetry = JobTelemetry(cpu=0.12345, memory=256.123, timestamp=timestamp)
@@ -1177,7 +1183,12 @@ class TestJobTelemetryFormatter:
     ) -> None:
         console = new_console(tty=True, color=True)
         with JobTelemetryFormatter(
-            console, datetime_formatter=format_datetime_human
+            console,
+            "owner",
+            parse_sort_keys("cpu"),
+            parse_top_columns(None),
+            image_formatter=str,
+            datetime_formatter=format_datetime_human,
         ) as fmt:
             timestamp = 1_517_248_466.238_723_6
             telemetry = JobTelemetry(cpu=0.12345, memory=256.123, timestamp=timestamp)
@@ -1201,7 +1212,12 @@ class TestJobTelemetryFormatter:
         job_descr2 = replace(job_descr, id=TEST_JOB_ID2)
         console = new_console(tty=True, color=True)
         with JobTelemetryFormatter(
-            console, datetime_formatter=format_datetime_human
+            console,
+            "owner",
+            parse_sort_keys("cpu"),
+            parse_top_columns(None),
+            image_formatter=str,
+            datetime_formatter=format_datetime_human,
         ) as fmt:
             timestamp = 1_517_248_466.238_723_6
             telemetry = JobTelemetry(cpu=0.12345, memory=256.123, timestamp=timestamp)
@@ -1219,6 +1235,16 @@ class TestJobTelemetryFormatter:
             assert not fmt.changed
             rich_cmp(console, index=1)
 
+            fmt.sort_keys = parse_sort_keys("owner,memory")
+            fmt.render()
+            assert not fmt.changed
+            rich_cmp(console, index=3)
+
+            fmt.sort_keys = parse_sort_keys("owner,-memory")
+            fmt.render()
+            assert not fmt.changed
+            rich_cmp(console, index=4)
+
             fmt.remove(job_descr2.id)
             assert fmt.changed
             fmt.render()
@@ -1231,7 +1257,13 @@ class TestJobTelemetryFormatter:
         job_descr2 = replace(job_descr, id=TEST_JOB_ID2)
         console = new_console(tty=True, color=True)
         with JobTelemetryFormatter(
-            console, maxrows=1, datetime_formatter=format_datetime_human
+            console,
+            "owner",
+            parse_sort_keys("cpu"),
+            parse_top_columns(None),
+            image_formatter=str,
+            datetime_formatter=format_datetime_human,
+            maxrows=1,
         ) as fmt:
             timestamp = 1_517_248_466.238_723_6
             telemetry = JobTelemetry(cpu=0.12345, memory=256.123, timestamp=timestamp)
@@ -1249,12 +1281,17 @@ class TestJobTelemetryFormatter:
             assert not fmt.changed
             rich_cmp(console, index=1)
 
-    def test_format_telemetry_line_with_gpu(
+    def test_format_telemetry_with_gpu(
         self, job_descr: JobDescription, rich_cmp: Any, new_console: _NewConsole
     ) -> None:
         console = new_console(tty=True, color=True)
         with JobTelemetryFormatter(
-            console, datetime_formatter=format_datetime_human
+            console,
+            "owner",
+            [],
+            parse_top_columns(None),
+            image_formatter=str,
+            datetime_formatter=format_datetime_human,
         ) as fmt:
             timestamp = 1_517_248_466
             telemetry = JobTelemetry(
@@ -1484,7 +1521,7 @@ class TestTabularJobsFormatter:
     def test_empty(self, rich_cmp: Any) -> None:
         formatter = TabularJobsFormatter(
             "owner",
-            parse_columns(None),
+            parse_ps_columns(None),
             image_formatter=str,
             datetime_formatter=format_datetime_human,
         )
@@ -1522,7 +1559,7 @@ class TestTabularJobsFormatter:
         )
         formatter = TabularJobsFormatter(
             "owner",
-            parse_columns(None),
+            parse_ps_columns(None),
             image_formatter=str,
             datetime_formatter=format_datetime_human,
         )
@@ -1600,7 +1637,7 @@ class TestTabularJobsFormatter:
         ]
         formatter = TabularJobsFormatter(
             "owner",
-            parse_columns(None),
+            parse_ps_columns(None),
             image_formatter=str,
             datetime_formatter=format_datetime_human,
         )
@@ -1632,7 +1669,7 @@ class TestTabularJobsFormatter:
             pass_config=True,
         )
 
-        columns = parse_columns("{status;align=right;min=20;Status Code}")
+        columns = parse_ps_columns("{status;align=right;min=20;Status Code}")
         formatter = TabularJobsFormatter(
             "owner",
             columns,
@@ -1672,7 +1709,7 @@ class TestTabularJobsFormatter:
             for i, life_span in enumerate(life_spans, 1)
         ]
 
-        columns = parse_columns("id life_span")
+        columns = parse_ps_columns("id life_span")
         formatter = TabularJobsFormatter(
             "owner",
             columns,
@@ -1737,7 +1774,7 @@ class TestTabularJobsFormatter:
             for i, item in enumerate(items, 1)
         ]
 
-        columns = parse_columns("id status when created started finished")
+        columns = parse_ps_columns("id status when created started finished")
         formatter = TabularJobsFormatter(
             "test-user",
             columns,
@@ -1777,7 +1814,7 @@ class TestTabularJobsFormatter:
             for i, working_dir in enumerate(items, 1)
         ]
 
-        columns = parse_columns("id workdir")
+        columns = parse_ps_columns("id workdir")
         formatter = TabularJobsFormatter(
             "test-user",
             columns,
@@ -1817,7 +1854,7 @@ class TestTabularJobsFormatter:
             for i, preset_name in enumerate(items, 1)
         ]
 
-        columns = parse_columns("id preset")
+        columns = parse_ps_columns("id preset")
         formatter = TabularJobsFormatter(
             "test-user",
             columns,
@@ -1825,3 +1862,87 @@ class TestTabularJobsFormatter:
             datetime_formatter=format_datetime_human,
         )
         rich_cmp(formatter(jobs))
+
+
+class TestLifeSpanUpdateFormatter:
+    async def test_not_finished(self, rich_cmp: Any) -> None:
+        job = JobDescription(
+            status=JobStatus.RUNNING,
+            owner="test-user",
+            cluster_name="default",
+            id=f"job-id",
+            uri=URL(f"job://default/test-user/job-id"),
+            description=None,
+            history=JobStatusHistory(
+                status=JobStatus.RUNNING,
+                reason="ErrorReason",
+                description="ErrorDesc",
+                created_at=isoparse("2018-09-25T12:28:21.298672+00:00"),
+                started_at=isoparse("2018-09-25T12:28:59.759433+00:00"),
+                finished_at=datetime.now(timezone.utc) - timedelta(seconds=1),
+                transitions=[
+                    JobStatusItem(
+                        status=JobStatus.PENDING,
+                        transition_time=isoparse("2018-09-25T12:28:21.298672+00:00"),
+                        reason="Creating",
+                    ),
+                    JobStatusItem(
+                        status=JobStatus.PENDING,
+                        transition_time=isoparse("2018-09-25T12:28:22.298672+00:00"),
+                        reason="Scheduling",
+                    ),
+                    JobStatusItem(
+                        status=JobStatus.PENDING,
+                        transition_time=isoparse("2018-09-25T12:28:23.298672+00:00"),
+                        reason="ContainerCreating",
+                    ),
+                    JobStatusItem(
+                        status=JobStatus.RUNNING,
+                        transition_time=isoparse("2018-09-25T12:28:24.759433+00:00"),
+                    ),
+                ],
+            ),
+            container=Container(
+                command="test-command",
+                image=RemoteImage.new_external_image(name="test-image"),
+                resources=Resources(16, 0.1, 0, None, False, None, None),
+            ),
+            scheduler_enabled=False,
+            pass_config=True,
+        )
+
+        formatter = LifeSpanUpdateFormatter(
+            datetime_formatter=format_datetime_human,
+        )
+        rich_cmp(formatter(job))
+
+    async def test_finished(self, rich_cmp: Any) -> None:
+        job = JobDescription(
+            status=JobStatus.SUCCEEDED,
+            owner="test-user",
+            cluster_name="default",
+            id=f"job-id",
+            uri=URL(f"job://default/test-user/job-id"),
+            description=None,
+            history=JobStatusHistory(
+                status=JobStatus.SUCCEEDED,
+                reason="ErrorReason",
+                description="ErrorDesc",
+                created_at=isoparse("2018-09-25T12:28:21.298672+00:00"),
+                started_at=isoparse("2018-09-25T12:28:59.759433+00:00"),
+                finished_at=datetime.now(timezone.utc) - timedelta(seconds=1),
+            ),
+            container=Container(
+                command="test-command",
+                image=RemoteImage.new_external_image(name="test-image"),
+                resources=Resources(16, 0.1, 0, None, False, None, None),
+            ),
+            scheduler_enabled=False,
+            pass_config=True,
+            life_span=3600,
+        )
+
+        formatter = LifeSpanUpdateFormatter(
+            datetime_formatter=format_datetime_human,
+        )
+        rich_cmp(formatter(job))
